/* Pending.c	-- Handle storage of unsent messages
 * $Id: Pending.c,v 1.1 2001/04/15 19:31:10 kvance Exp $
 * Copyright (C) 2000-2001 Kev Vance <kvance@tekktonik.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <PalmOS.h>
#include "PalmLJ.h"

UInt16 ljErr;

Entry *CreateEntry(Char *user, Char *uj, Char *passwd, Char *subject,
		Char *text, UInt16 moodnumber, Char *mood, DateTimeType date)
{
	/* Allocate memory */
	Entry *e = MemPtrNew(sizeof(Entry));

	/* Copy everything */
	e->version = 1;
	MemMove(&e->date, &date, sizeof(DateTimeType));
	e->userLength = StrLen(user)+1;
	e->user = MemPtrNew(e->userLength);
	StrCopy(e->user, user);
	if(uj == NULL) {
		e->ujLength = 0;
		e->uj = NULL;
	} else {
		e->ujLength = StrLen(uj)+1;
		e->uj = MemPtrNew(e->ujLength);
		StrCopy(e->uj, uj);
	}
	e->passwdLength = StrLen(passwd)+1;
	e->passwd = MemPtrNew(e->passwdLength);
	StrCopy(e->passwd, passwd);
	if(subject == NULL) {
		e->subjLength = 0;
		e->subject = NULL;
	} else {
		e->subjLength = StrLen(subject)+1;
		e->subject = MemPtrNew(e->subjLength);
		StrCopy(e->subject, subject);
	}
	e->textLength = StrLen(text)+1;
	e->text = MemPtrNew(e->textLength);
	StrCopy(e->text, text);
	e->moodNumber = moodnumber;
	if(mood == NULL) {
		e->moodLength = 0;
		e->mood = NULL;
	} else {
		e->moodLength = StrLen(mood)+1;
		e->mood = MemPtrNew(e->moodLength);
		StrCopy(e->mood, mood);
	}

	return e;
}

void DeleteEntry(Entry *e)
{
	if(e->user != NULL)
		MemPtrFree(e->user);
	if(e->uj != NULL)
		MemPtrFree(e->uj);
	if(e->passwd != NULL)
		MemPtrFree(e->passwd);
	if(e->subject != NULL)
		MemPtrFree(e->subject);
	if(e->text != NULL)
		MemPtrFree(e->text);
	if(e->mood != NULL)
		MemPtrFree(e->mood);
	MemPtrFree(e);
}

int SaveEntry(Entry *e)
{
	LocalID id;
	DmOpenRef db;
	
	MemHandle record;
	Char *raw;
	UInt32 length;
	UInt16 position;
	UInt32 ofs = 0;

	/* Open or create entries database */
	id = DmFindDatabase(0, "PalmLJ Entries");
	if(id == 0) {
		DmCreateDatabase(0, "PalmLJ Entries", idCreator, idDatabase,
				false);
		id = DmFindDatabase(0, "PalmLJ Entries");
		if(id == 0)
			return 0;
	}
	db = DmOpenDatabase(0, id, dmModeReadWrite);

	/* Create entry */
	length =  	/*version*/	2 +
			/*userLength*/	2 +
			/*user*/	e->userLength +
			/*ujLength*/	2 +
			/*uj*/		e->ujLength +
			/*passwdLength*/2 +
			/*passwd*/	e->passwdLength +
			/*subjLength*/	2 +
			/*subject*/	e->subjLength +
			/*textLength*/	2 +
			/*text*/	e->textLength +
			/*moodNumber*/	2 +
			/*moodLength*/	2 +
			/*mood*/	e->moodLength +
			/*date*/	sizeof(DateTimeType);
	position = dmMaxRecordIndex;
	record = DmNewRecord(db, &position, length);
	if(record == NULL) {
		DmCloseDatabase(db);
		return 0;
	}

	raw = MemHandleLock(record);
	DmWrite(raw, ofs, &e->version, 2);
	ofs += 2;
	DmWrite(raw, ofs, &e->userLength, 2);
	ofs += 2;
	DmWrite(raw, ofs, e->user, e->userLength);
	ofs += e->userLength;
	DmWrite(raw, ofs, &e->ujLength, 2);
	ofs += 2;
	if(e->ujLength != 0) {
		DmWrite(raw, ofs, e->uj, e->ujLength);
		ofs += e->ujLength;
	}
	DmWrite(raw, ofs, &e->passwdLength, 2);
	ofs += 2;
	DmWrite(raw, ofs, e->passwd, e->passwdLength);
	ofs += e->passwdLength;
	DmWrite(raw, ofs, &e->subjLength, 2);
	ofs += 2;
	if(e->subjLength != 0) {
		DmWrite(raw, ofs, e->subject, e->subjLength);
		ofs += e->subjLength;
	}
	DmWrite(raw, ofs, &e->textLength, 2);
	ofs += 2;
	DmWrite(raw, ofs, e->text, e->textLength);
	ofs += e->textLength;
	DmWrite(raw, ofs, &e->moodNumber, 2);
	ofs += 2;
	DmWrite(raw, ofs, &e->moodLength, 2);
	ofs += 2;
	if(e->moodLength != 0) {
		DmWrite(raw, ofs, e->mood, e->moodLength);
		ofs += e->moodLength;
	}
	DmWrite(raw, ofs, &e->date, sizeof(DateTimeType));
	ofs += sizeof(DateTimeType);
	MemHandleUnlock(record);

	DmReleaseRecord(db, position, true);
	DmCloseDatabase(db);

	return -1;
}

int SaveUsers(UserRecord *base)
{
	UInt16 version = 1;
	LocalID id;
	DmOpenRef db;
	
	MemHandle record;
	Char *raw;
	UInt32 length;
	UInt16 position;
	UInt32 ofs = 0;

	UserRecord *r;
	AccessRecord *a;
	int i, j;

	/* We create a new database every time */
	id = DmFindDatabase(0, "PalmLJ Users");
	if(id != 0)
		DmDeleteDatabase(0, id);
	DmCreateDatabase(0, "PalmLJ Users", idCreator, idDatabase, false);
	id = DmFindDatabase(0, "PalmLJ Users");
	if(id == 0)
		return 0;
	db = DmOpenDatabase(0, id, dmModeReadWrite);

	/* Create entry */
	length =	/* version */	2 +
			/* users */	2;
	length += (	/* user */	64 +
			/* passwd */	32 +
			/* name */	128 +
			/*accessCount*/ 2
		  ) * userCount;
	for(i = 0; i < userCount; i++) {
		r = GetUserRecord(base, i);
		length += 64 * r->accessCount;
	}

	position = dmMaxRecordIndex;
	record = DmNewRecord(db, &position, length);
	if(record == NULL) {
		DmCloseDatabase(db);
		return 0;
	}

	raw = MemHandleLock(record);
	DmWrite(raw, ofs, &version, 2);
	ofs += 2;
	DmWrite(raw, ofs, &userCount, 2);
	ofs += 2;
	for(i = 0; i < userCount; i++) {
		r = GetUserRecord(base, i);
		DmWrite(raw, ofs, r->user, 64);
		ofs += 64;
		DmWrite(raw, ofs, r->passwd, 32);
		ofs += 32;
		DmWrite(raw, ofs, r->name, 128);
		ofs += 128;
		DmWrite(raw, ofs, &r->accessCount, 2);
		ofs += 2;
		for(j = 0; j < r->accessCount; j++) {
			a = GetAccessRecord(r->accessRecord, j);
			DmWrite(raw, ofs, a->user, 64);
			ofs += 64;
		}
	}

	MemHandleUnlock(record);

	DmReleaseRecord(db, position, true);
	DmCloseDatabase(db);

	return -1;
}

UserRecord *GetUsers(void)
{
	UInt16 version = 1;
	LocalID id;
	DmOpenRef db;
	
	MemHandle record;
	Char *raw;
	UInt16 position;
	UInt32 ofs = 0;

	UserRecord *r = NULL, *base = NULL;
	AccessRecord *a = NULL;
	int i, j;

	/* We create a new database every time */
	id = DmFindDatabase(0, "PalmLJ Users");
	if(id == 0) {
		ljErr = 0;
		return NULL;
	}
	db = DmOpenDatabase(0, id, dmModeReadOnly);

	/* Get records */
	record = DmGetRecord(db, 0);
	raw = MemHandleLock(record);
	if(raw == NULL) {
		DmCloseDatabase(db);
		ljErr = 1;
		return NULL;
	}

	/* Parse */
	MemMove(&version, &raw[ofs], 2);
	ofs += 2;
	if(version != 1) {
		MemHandleUnlock(record);
		DmReleaseRecord(db, position, true);
		DmCloseDatabase(db);
		ljErr = 1;
		return NULL;
	}
	MemMove(&userCount, &raw[ofs], 2);
	ofs += 2;

	for(i = 0; i < userCount; i++) {
		if(base == NULL) {
			r = MemPtrNew(sizeof(UserRecord));
			base = r;
		} else {
			r->next = MemPtrNew(sizeof(UserRecord));
			r = r->next;
		}
		r->accessRecord = NULL;
		r->next = NULL;
		MemMove(r->user, &raw[ofs], 64);
		ofs += 64;
		MemMove(r->passwd, &raw[ofs], 32);
		ofs += 32;
		MemMove(r->name, &raw[ofs], 128);
		ofs += 128;
		MemMove(&r->accessCount, &raw[ofs], 2);
		ofs += 2;
		for(j = 0; j < r->accessCount; j++) {
			if(r->accessRecord == NULL) {
				r->accessRecord = MemPtrNew(sizeof(
							AccessRecord));
				a = r->accessRecord;
			} else {
				a->next = MemPtrNew(sizeof(AccessRecord));
				a = a->next;
			}
			a->next = NULL;
			MemMove(a->user, &raw[ofs], 64);
			ofs += 64;
		}
	}

	MemHandleUnlock(record);

	DmReleaseRecord(db, position, true);
	DmCloseDatabase(db);

	ljErr = 0;
	return base;
}
