/* ljsync.c     -- "Conduit" for PalmLJ
 * $Id: ljsync.c,v 1.1 2001/04/15 19:31:10 kvance Exp $
 * Copyright (C) 2001 Kev Vance <kvance@tekktonik.net>
 * Based on memos.c (C) 1996 Kenneth Albanowski
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <unistd.h>

#include <pi-source.h>
#include <pi-socket.h>
#include <pi-dlp.h>

#include "palmlj.h"

UserRecord *userRecords;

/* fix endian-ness */
#define DELOUSE(e)	e = ntohs(e)
#define DATEDELOUSE(d)	d.second = ntohs(d.second);\
			d.minute = ntohs(d.minute);\
			d.hour = ntohs(d.hour);\
			d.day = ntohs(d.day);\
			d.month = ntohs(d.month);\
			d.year = ntohs(d.year);\
			d.weekDay = ntohs(d.weekDay)
/*
 * SHOULD INCLUDE HEADERS FOR mkdir()
 */

#define PILOTPORT "/dev/pilot"

char *progname;

int main(int argc, char *argv[]);

void usage(void)
{
	fprintf(stderr, "Usage: %s [options]\n", progname);
	fprintf(stderr, "\n");
	fprintf(stderr, "Options:\n");
	fprintf(stderr, "  -q          = do not prompt for HotSync button press\n");
	fprintf(stderr, "  -p port     = use device file <port> to communicate with Pilot\n");
	fprintf(stderr, "  -h|-?       = print this help\n");
	fprintf(stderr, "\n");
	fprintf(stderr, "ljsync will send all your posts to livejournal.\n");
	exit(0);
}

int main(int argc, char *argv[])
{
	struct pi_sockaddr addr;
	int db;
	int sd;
	int i;
	struct PilotUser U;
	int ret;
	unsigned char buffer[0xffff];
	char appblock[0xffff];

	int quiet = 0;
	int c;
	extern char *optarg;
	extern int optind;

	Entry entry;
	int ofs = 0;
	recordid_t id;

	int successes = 0;
	int errors = 0;

	progname = argv[0];

	if (getenv("PILOTPORT")) {
		strcpy(addr.pi_device, getenv("PILOTPORT"));
	} else {
		strcpy(addr.pi_device, PILOTPORT);
	}

	while (((c = getopt(argc, argv, "qp:h?")) != -1)) {
		switch (c) {
		case 'q':
			quiet = 1;
			break;
		case 'p':
			/* optarg is name of port to use instead of $PILOTPORT or /dev/pilot */
			strcpy(addr.pi_device, optarg);
			break;
		case 'h':
		case '?':
			usage();
		}
	}

	if (!quiet) {
		fprintf(stderr,
			"Please insert Pilot in cradle on %s and press HotSync button.\n",
			addr.pi_device);
	}
	if (!(sd = pi_socket(PI_AF_SLP, PI_SOCK_STREAM, PI_PF_PADP))) {
		perror("pi_socket");
		exit(1);
	}
	addr.pi_family = PI_AF_SLP;

	ret = pi_bind(sd, (struct sockaddr *) &addr, sizeof(addr));
	if (ret == -1) {
		perror("pi_bind");
		exit(1);
	}
	ret = pi_listen(sd, 1);
	if (ret == -1) {
		perror("pi_listen");
		exit(1);
	}
	sd = pi_accept(sd, 0, 0);
	if (sd == -1) {
		perror("pi_accept");
		exit(1);
	}
	/* Ask the pilot who it is. */
	dlp_ReadUserInfo(sd, &U);

	/* Tell user (via Pilot) that we are starting things up */
	dlp_OpenConduit(sd);

	/* Open the Memo Pad's database, store access handle in db */
	if (dlp_OpenDB(sd, 0, 0x80 | 0x40, "PalmLJ Entries", &db) < 0) {
		puts("Unable to open PalmLJ Entries");
		puts("Perhaps you haven't made any entries yet.");
		dlp_AddSyncLogEntry(sd, "Unable to open PalmLJ Entries.\n");
		exit(1);
	}
	dlp_ReadAppBlock(sd, db, 0, (unsigned char *) appblock, 0xffff);

	for (i = 0;; i++) {
		int attr;
		int category;

		int len = dlp_ReadRecordByIndex(sd, db, i, buffer, &id, 0, &attr, &category);
		if (len < 0) {
			if (i != 0) {
				/* make another pass */
				i = -1;
				dlp_CloseDB(sd, db);
				dlp_OpenDB(sd, 0, 0x80 | 0x40, "PalmLJ Entries",
					   &db);
				continue;
			} else
				break;
		}
		if (len < 2) {
			fprintf(stderr, "Entry has bad length!\n");
			continue;
		}
		memcpy(&entry.version, &buffer[ofs], 2);
		DELOUSE(entry.version);
		ofs += 2;
		if (entry.version != 1) {
			fprintf(stderr, "Entry has version %i, can't read!\n",
				entry.version);
			continue;
		}
		memcpy(&entry.userLength, &buffer[ofs], 2);
		DELOUSE(entry.userLength);
		ofs += 2;
		entry.user = &buffer[ofs];
		ofs += entry.userLength;
		memcpy(&entry.ujLength, &buffer[ofs], 2);
		DELOUSE(entry.ujLength);
		ofs += 2;
		if (entry.ujLength != 0) {
			entry.uj = &buffer[ofs];
			ofs += entry.ujLength;
		}
		memcpy(&entry.passwdLength, &buffer[ofs], 2);
		DELOUSE(entry.passwdLength);
		ofs += 2;
		entry.passwd = &buffer[ofs];
		ofs += entry.passwdLength;
		memcpy(&entry.subjLength, &buffer[ofs], 2);
		DELOUSE(entry.subjLength);
		ofs += 2;
		if (entry.subjLength != 0) {
			entry.subject = &buffer[ofs];
			ofs += entry.subjLength;
		}
		memcpy(&entry.textLength, &buffer[ofs], 2);
		DELOUSE(entry.textLength);
		ofs += 2;
		entry.text = &buffer[ofs];
		ofs += entry.textLength;
		memcpy(&entry.moodNumber, &buffer[ofs], 2);
		DELOUSE(entry.moodNumber);
		ofs += 2;
		memcpy(&entry.moodLength, &buffer[ofs], 2);
		DELOUSE(entry.moodLength);
		ofs += 2;
		if (entry.moodLength != 0) {
			entry.mood = &buffer[ofs];
			ofs += entry.moodLength;
		}
		memcpy(&entry.date, &buffer[ofs], sizeof(DateTimeType));
		ofs += sizeof(DateTimeType);
		DATEDELOUSE(entry.date);
		printf("Entry by %s ", entry.user);
		if (entry.ujLength != 0)
			printf("as %s ", entry.uj);
		printf("(%i/%i/%i %i:%02i)\n\t%s\n", entry.date.month,
		       entry.date.day, entry.date.year,
		       entry.date.hour, entry.date.minute,
		       entry.subjLength == 0 ? "(no subject)" :
		       entry.subject);
		if (Post(&entry)) {
			printf("\tPosted.\n");
			dlp_DeleteRecord(sd, db, 0, id);
			successes++;
		} else {
			printf("\tError!\n");
			errors++;
			/* flush excess bogons */
			sleep(1);
		}
		fflush(stdout);
		ofs = 0;
	}

	/* Close the database */
	dlp_CloseDB(sd, db);

	dlp_AddSyncLogEntry(sd, "Read entries from Pilot.\n");

	pi_close(sd);

	printf("ljsync finished.  %i success%s %i error%s\n", successes,
	       successes == 1 ? "," : "es,", errors,
	       errors == 1 ? "." : "s.");

	return 0;
}
