/* ljusers.c    -- "Conduit" for PalmLJ
 * $Id: ljusers.c,v 1.1 2001/04/15 19:31:10 kvance Exp $
 * Copyright (C) 2001 Kev Vance <kvance@tekktonik.net>
 * Based on memos.c (C) 1996 Kenneth Albanowski
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <unistd.h>

#include <pi-source.h>
#include <pi-socket.h>
#include <pi-dlp.h>

#include <netinet/in.h>

#include "palmlj.h"
#include "md5.h"

#define PILOTPORT "/dev/pilot"

char *progname;
UserRecord *userRecords = NULL;

void usage(void)
{
	fprintf(stderr, "Usage: %s [options]\n", progname);
	fprintf(stderr, "\n");
	fprintf(stderr, "Options:\n");
	fprintf(stderr, "  -a user     = add a user named <user> (you will be promped for a password)\n");
	fprintf(stderr, "  -d user     = delete a user named <user>\n");
	fprintf(stderr, "  -l          = list users\n");
	fprintf(stderr, "  -q          = do not prompt for HotSync button press\n");
	fprintf(stderr, "  -p port     = use device file <port> to communicate with Pilot\n");
	fprintf(stderr, "  -h|-?       = print this help\n");
	fprintf(stderr, "\n");
	exit(0);
}

enum modes {
	Undecided, Add, Delete, List
};

int main(int argc, char *argv[])
{
	struct pi_sockaddr addr;
	int db = 0;
	int sd;
	int i, j;
	struct PilotUser U;
	int ret;
	unsigned char buffer[0xffff];
	char appblock[0xffff];

	int quiet = 0;
	int attr, category, len;
	int c;
	extern char *optarg;
	extern int optind;

	int mode = Undecided;
	char *user;
	char passwd[64];
	char hpasswd[32];
	unsigned char digest[16];
	struct md5_ctx ctx;

	recordid_t id;

	UserRecord *r = NULL, *s = NULL;
	AccessRecord *a = NULL;
	u_int16_t version;
	u_int16_t userCount;

	int ofs = 0;
	int success = 0;

	progname = argv[0];

	if (getenv("PILOTPORT")) {
		strcpy(addr.pi_device, getenv("PILOTPORT"));
	} else {
		strcpy(addr.pi_device, PILOTPORT);
	}

	while (((c = getopt(argc, argv, "qp:d:h?a:d:l")) != -1)) {
		switch (c) {
		case 'q':
			quiet = 1;
			break;
		case 'a':
			mode = Add;
			user = strdup(optarg);
			break;
		case 'd':
			mode = Delete;
			user = strdup(optarg);
			break;
		case 'l':
			mode = List;
			break;
		case 'p':
			/* optarg is name of port to use instead of $PILOTPORT or /dev/pilot */
			strcpy(addr.pi_device, optarg);
			break;
		case 'h':
		case '?':
			usage();
		}
	}

	if (mode == Undecided) {
		usage();
		exit(0);
	}
	if (mode == Add) {
		/* Login & add the specified user */
		printf("Enter password for %s: ", user);
		system("stty -echo");
		fgets(passwd, 64, stdin);
		printf("\n");
		system("stty echo");
		/* strip newline */
		passwd[strlen(passwd) - 1] = '\0';
		/* md5 it */
		md5_init_ctx(&ctx);
		md5_process_bytes(passwd, strlen(passwd), &ctx);
		md5_finish_ctx(&ctx, digest);
		for (i = 0; i < 16; i++)
			sprintf(&hpasswd[i * 2], "%02x", digest[i]);
	}
	if (!quiet) {
		fprintf(stderr,
			"Please insert Pilot in cradle on %s and press HotSync button.\n",
			addr.pi_device);
	}
	if (!(sd = pi_socket(PI_AF_SLP, PI_SOCK_STREAM, PI_PF_PADP))) {
		perror("pi_socket");
		exit(1);
	}
	addr.pi_family = PI_AF_SLP;

	ret = pi_bind(sd, (struct sockaddr *) &addr, sizeof(addr));
	if (ret == -1) {
		perror("pi_bind");
		exit(1);
	}
	ret = pi_listen(sd, 1);
	if (ret == -1) {
		perror("pi_listen");
		exit(1);
	}
	sd = pi_accept(sd, 0, 0);
	if (sd == -1) {
		perror("pi_accept");
		exit(1);
	}
	/* Ask the pilot who it is. */
	dlp_ReadUserInfo(sd, &U);

	/* Tell user (via Pilot) that we are starting things up */
	dlp_OpenConduit(sd);

	/* Open the Memo Pad's database, store access handle in db */
	if (dlp_OpenDB(sd, 0, 0x80 | 0x40, "PalmLJ Users", &db) < 0) {
		if (mode != Add) {
			puts("There are no users defined yet.");
			dlp_AddSyncLogEntry(sd, "Unable to open PalmLJ Users.\n");
			exit(1);
		} else {
			goto _no_need_to_read;
			/* Shut up :P */
		}
	}
	dlp_ReadAppBlock(sd, db, 0, (unsigned char *) appblock, 0xffff);

	len = dlp_ReadRecordByIndex(sd, db, 0, buffer, &id, 0, &attr, &category);
	if (len < 4) {
		fprintf(stderr, "Invalid LJ Users file!\n" \
			"Try deleting PalmLJ and reinstalling.\n");
		dlp_CloseDB(sd, db);
		dlp_AddSyncLogEntry(sd, "Bad users database encountered!\n");
		pi_close(sd);
		exit(1);
	}
	memcpy(&version, &buffer[ofs], 2);
	version = ntohs(version);
	ofs += 2;
	if (version != 1) {
		fprintf(stderr, "We can only parse version 1 files!\n");
		dlp_CloseDB(sd, db);
		dlp_AddSyncLogEntry(sd, "Old sync client, sorry :P");
		pi_close(sd);
		exit(1);
	}
	if (mode == List)
		printf("\nPalmLJ User Records\n-------------------\n");

	memcpy(&userCount, &buffer[ofs], 2);
	userCount = ntohs(userCount);
	ofs += 2;
	if (userCount == 0 && mode == List)
		printf("(none)\n");

	for (i = 0; i < userCount; i++) {
		if (userRecords == NULL) {
			userRecords = malloc(sizeof(UserRecord));
			r = userRecords;
		} else {
			r->next = malloc(sizeof(UserRecord));
			r = r->next;
		}
		r->next = NULL;
		r->accessRecord = NULL;
		memcpy(r->user, &buffer[ofs], 64);
		ofs += 64;
		memcpy(r->passwd, &buffer[ofs], 32);
		ofs += 32;
		memcpy(r->name, &buffer[ofs], 128);
		ofs += 128;
		memcpy(&r->accessCount, &buffer[ofs], 2);
		r->accessCount = ntohs(r->accessCount);
		ofs += 2;
		for (j = 0; j < r->accessCount; j++) {
			if (r->accessRecord == NULL) {
				r->accessRecord = malloc(sizeof(
							  AccessRecord));
				a = r->accessRecord;
			} else {
				a->next = malloc(sizeof(AccessRecord));
				a = a->next;
			}
			a->next = NULL;
			memcpy(a->user, &buffer[ofs], 64);
			ofs += 64;
		}
		if (mode == List)
			printf("%s (%s)\n", r->user, r->name);
	}

	if (mode == List) {
		/* Say good night */
		dlp_CloseDB(sd, db);
		dlp_AddSyncLogEntry(sd, "Read PalmLJ users.  Bye.\n");
		pi_close(sd);
		exit(0);
	}
      _no_need_to_read:
	if (mode == Add) {
		if (!Login(user, hpasswd, &userCount)) {
			/* Login failed */
			dlp_CloseDB(sd, db);
			dlp_AddSyncLogEntry(sd, "PalmLJ login failed :P\n");
			pi_close(sd);
			fprintf(stderr, "Login failed.\n");
			exit(1);
		}
		r = GetUserRecord(userRecords, userCount - 1);
		printf("\nWelcome to LiveJournal, %s.\n", r->name);
		if (db == 0) {
			/* Gotta create the DB */
			dlp_CreateDB(sd, idCreator, idDatabase, 0, 0, 1,
				     "PalmLJ Users", &db);
			version = 1;
		}
	}
	if (mode == Delete) {
		/* Search for & delete the specified user */
		for (i = 0; i < userCount; i++) {
			r = GetUserRecord(userRecords, i);
			if (!strcmp(r->user, user)) {
				if (i == 0) {
					r = DelUserRecord(userRecords);
					userRecords = r;
					userCount--;
				} else {
					r = GetUserRecord(userRecords, i - 1);
					s = GetUserRecord(userRecords, i);
					s = DelUserRecord(s);
					r->next = s;
					userCount--;
				}
				success = 1;
			}
		}
		if (success == 1) {
			printf("User %s deleted.\n", user);
		} else {
			printf("No such user %s exists!\n", user);
		}
		if (userCount == 0) {
			/* Don't need the DB anymore */
			dlp_CloseDB(sd, db);
			dlp_DeleteDB(sd, 0, "PalmLJ Users");
			pi_close(sd);
			exit(0);
		}
	}
	/* Create new record w/ changes */
	len = /* version */ 2 +
	/* users */ 2;
	len += ( /* user */ 64 +
	/* passwd */ 32 +
	/* name */ 128 +
	/*accessCount */ 2
	    ) * userCount;
	for (i = 0; i < userCount; i++) {
		r = GetUserRecord(userRecords, i);
		len += 64 * r->accessCount;
	}
	ofs = 0;
	version = htons(version);
	memcpy(&buffer[ofs], &version, 2);
	ofs += 2;
	userCount = htons(userCount);
	memcpy(&buffer[ofs], &userCount, 2);
	userCount = ntohs(userCount);
	ofs += 2;
	for (i = 0; i < userCount; i++) {
		r = GetUserRecord(userRecords, i);
		memcpy(&buffer[ofs], r->user, 64);
		ofs += 64;
		memcpy(&buffer[ofs], r->passwd, 32);
		ofs += 32;
		memcpy(&buffer[ofs], r->name, 128);
		ofs += 128;
		r->accessCount = htons(r->accessCount);
		memcpy(&buffer[ofs], &r->accessCount, 2);
		ofs += 2;
		r->accessCount = ntohs(r->accessCount);
		for (j = 0; j < r->accessCount; j++) {
			a = GetAccessRecord(r->accessRecord, j);
			memcpy(&buffer[ofs], a->user, 64);
			ofs += 64;
		}
	}

	/* Send new record */
	dlp_WriteRecord(sd, db, 0, id, 0, buffer, len, &id);

	/* Close the database */
	dlp_CloseDB(sd, db);

	dlp_AddSyncLogEntry(sd, "FIXME, say something here.\n");

	pi_close(sd);

	return 0;
}
