/* net.c	-- Network code from PalmLJ, unixified
 * $Id: net.c,v 1.1 2001/04/15 19:31:10 kvance Exp $
 * Copyright (C) 2000-2001 Kev Vance <kvance@tekktonik.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* This is probably the only time anyone's ported from Palm to UNIX :P */

#include <netinet/in.h>
#include <sys/select.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <sys/time.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <netdb.h>

#include "palmlj.h"

size_t bytes;

void UrlCat(char * x, char * y)
{
    int i, j, hex;

    j = strlen(x);		// The end of the dest. string

    for (i = 0; i < strlen(y); i++) {
	if (bytes < j + 4) {
	    /* Out of memory, request more */
	    bytes += 100;	
	    realloc(x, bytes);
	}
	if ((y[i] >= 'a' && y[i] <= 'z') ||	// a-z
	     (y[i] >= 'A' && y[i] <= 'Z') ||	// A-Z
	     (y[i] >= '0' && y[i] <= '9')) {	// 0-9

	    x[j++] = y[i];
	    x[j] = '\0';
	} else if (y[i] == ' ') {
	    x[j++] = '+';
	    x[j] = '\0';
	} else {
	    x[j++] = '%';
	    /* Left hex digit */
	    hex = y[i] / 16;
	    if (hex < 10)
		hex += '0';
	    else
		hex += 'A' - 10;
	    x[j++] = hex;
	    /* Right hex digit */
	    hex = y[i] % 16;
	    if (hex < 10)
		hex += '0';
	    else
		hex += 'A' - 10;
	    x[j++] = hex;
	    x[j] = '\0';
	}
    }
}

void MemCat(char * x, char * y)
{
    int i, j;

    j = strlen(x);		// The end of the dest. string

    for (i = 0; i < strlen(y); i++) {
	if (bytes < j + 2) {
	    /* Out of memory, request more */
	    bytes += 100;
	    realloc(x, bytes);
	}
	x[j++] = y[i];
	x[j] = '\0';
    }
}

void MemCatL(char * x, char * y, int l)
{
    int i, j;

    j = strlen(x);		// The end of the dest. string

    for (i = 0; i < l; i++) {
	if (bytes < j + 2) {
	    /* Out of memory, request more */
	    bytes += 100;	
	    realloc(x, bytes);
	}
	x[j++] = y[i];
	x[j] = '\0';
    }
}

int Post(Entry *e)
{
    Char *dest;
    Char *buffer;

    int port = 80;
    int sock;
    struct hostent *hp;
    struct sockaddr_in sin;

    /* Lookup/Connect to the LJ server */
    if((hp = gethostbyname("www.livejournal.com")) == NULL)
	    return 0;
    memcpy(&sin.sin_addr, hp->h_addr, hp->h_length);
    sin.sin_family = hp->h_addrtype;
    sin.sin_port = htons(port);
    if((sock = socket(hp->h_addrtype, SOCK_STREAM, 0)) < 0)
	    return 0;
    if(connect(sock, (struct sockaddr *)&sin, sizeof(sin)) < 0)
	    return 0;

    /* Send HTTP headers */
    send(sock, "POST /cgi-bin/log.cgi HTTP/1.0\nHost: www.livejournal.com\nContent-type: multipart/form-data\nContent-length: ", 107, 0);

    /* Allocate some memory */
    dest = malloc(5000);
    buffer = malloc(1024);
    if (dest == NULL) {
	close(sock);
	return 0;
    }

    /* Build the HTTP POST request */
    bytes = 5000;
    strcpy(dest, "mode=postevent&user=");
    UrlCat(dest, e->user);
    MemCat(dest, "&hpassword=");
    MemCatL(dest, e->passwd, 32);
    MemCat(dest, "&event=");
    UrlCat(dest, e->text);
    MemCat(dest, "&lineendings=unix&subject=");
    if (e->subject != NULL)
	UrlCat(dest, e->subject);
    MemCat(dest, "&year=");
    sprintf(buffer, "%i", e->date.year);
    MemCat(dest, buffer);
    MemCat(dest, "&mon=");
    sprintf(buffer, "%i", e->date.month);
    MemCat(dest, buffer);
    MemCat(dest, "&day=");
    sprintf(buffer, "%i", e->date.day);
    MemCat(dest, buffer);
    MemCat(dest, "&hour=");
    sprintf(buffer, "%i", e->date.hour);
    MemCat(dest, buffer);
    MemCat(dest, "&min=");
    sprintf(buffer, "%i", e->date.minute);
    MemCat(dest, buffer);
    if (e->moodNumber == 1) {
	MemCat(dest, "&prop_current_mood=");
	UrlCat(dest, e->mood);
    } else if (e->moodNumber != 0) {
	MemCat(dest, "&prop_current_moodid=");
	sprintf(buffer, "%i", e->moodNumber - 2);
	MemCat(dest, buffer);
    }
    if (e->uj != NULL) {
	MemCat(dest, "&usejournal=");
	UrlCat(dest, e->uj);
    }
    MemCat(dest, "\n");

    sprintf(buffer, "%i", strlen(dest) - 1);
    send(sock, buffer, strlen(buffer), 0);
    send(sock, "\n\n", 2, 0);
    send(sock, dest, strlen(dest), 0);

    /* Good enough for me */
    close(sock);

    free(dest);
    free(buffer);

    return -1;
}

int Login(char * user, char * passwd, u_int16_t *userCount)
{

    Char *dest;
    Char *dp;
    Char *buffer;
    Int16 i, acount;
    u_int16_t uc;

    int searchstate = 0;
    int success;

    UserRecord *r = NULL;
    AccessRecord *a = NULL, *b, *c;

    int port = 80;
    int sock;
    struct hostent *hp;
    struct sockaddr_in sin;

    struct timeval tv;
    fd_set fds_read;

#define R_NAME		1
#define R_ACCESS	2

    /* Lookup/Connect to the LJ server */
    if((hp = gethostbyname("www.livejournal.com")) == NULL)
	    return 0;
    memcpy(&sin.sin_addr, hp->h_addr, hp->h_length);
    sin.sin_family = hp->h_addrtype;
    sin.sin_port = htons(port);
    if((sock = socket(hp->h_addrtype, SOCK_STREAM, 0)) < 0)
	    return 0;
    if(connect(sock, (struct sockaddr *)&sin, sizeof(sin)) < 0)
	    return 0;

    /* Send HTTP headers */
    send(sock, "POST /cgi-bin/log.cgi HTTP/1.0\nHost: www.livejournal.com\nContent-type: multipart/form-data\nContent-length: ", 107, 0);

    /* Allocate some memory */
    dest = malloc(5000);
    buffer = malloc(256);
    if (dest == NULL) {
	close(sock);
	return 0;
    }
    /* Build the HTTP POST request */
    bytes = 5000;
    strcpy(dest, "mode=login&user=");
    UrlCat(dest, user);
    MemCat(dest, "&hpassword=");
    MemCatL(dest, passwd, 32);
    MemCat(dest, "&clientversion=PalmOS-PalmLJ/1.2.0");
#if 0 /* Not implemented yet */
    if (moods == true) {
	MemCat(dest, "&getmoods=");
	StrIToA(buffer, prefs->moodCount);
	MemCat(dest, buffer);
    }
#endif
    MemCat(dest, "\n");

    sprintf(buffer, "%i", strlen(dest) - 1);
    send(sock, buffer, strlen(buffer), 0);
    send(sock, "\n\n", 2, 0);
    send(sock, dest, strlen(dest), 0);

    /* Receive something */
    FD_ZERO(&fds_read);
    FD_SET(sock, &fds_read);
    tv.tv_sec = 10;
    tv.tv_usec= 0;
    if(select(sock+1, &fds_read, NULL, NULL, &tv) < 1) {
	    /* timeout */
	    return 0;
    }
    i = read(sock, dest, 5000);

    dest[i] = '\0';
    dp = strstr(dest, "\r\n\r\n");
    if(dp == NULL)
	    /* Where's my newlines, jerk?! */
	    return 0;
    dp += 4;

    /* Close the socket/library */
    close(sock);

    success = 0;
    acount = 0;
    /* Add the user */
    for (i = 0; i < 256; i++) {
	if (dp[i] == '\n') {
	    /* end of line */
	    dp[i] = '\0';
	    if (searchstate == R_NAME) {
		/* We got the name!  */
		if (*userCount == 0) {
		    userRecords = malloc(sizeof(UserRecord));
		    r = userRecords;
		} else {
		    r = GetUserRecord(userRecords, *userCount - 1);
		    r->next = malloc(sizeof(UserRecord));
		    r = r->next;
		}
		uc = *userCount + 1;
		memcpy(userCount, &uc, 2);
		r->accessCount = 0;
		r->accessRecord = NULL;
		r->next = NULL;
		memcpy(r->user, user, strlen(user) + 1);
		memcpy(r->passwd, passwd, 32);
		memcpy(r->name, dp, strlen(dp) + 1);
		dp[i++] = '\n';
		success = -1;
		if (a != NULL) {
		    /* We also have some access records */
		    r->accessRecord = a;
		    r->accessCount = acount;
		}
		break;
	    } else if (searchstate == R_ACCESS) {
		/* Get access record */
		acount++;
		if (a == NULL) {
		    a = malloc(sizeof(AccessRecord));
		    memcpy(a->user, dp, strlen(dp) + 1);
		    a->next = NULL;
		} else {
		    b = GetAccessRecord(a, acount - 2);
		    c = malloc(sizeof(AccessRecord));
		    memcpy(c->user, dp, strlen(dp) + 1);
		    b->next = c;
		}
		if (r != NULL) {
		    /* If we have a name, write to it */
		    r->accessRecord = a;
		    r->accessCount = acount;
		}
		searchstate = 0;
		dp[i++] = '\n';
		dp += i;
		i = 0;
	    } else if (!strcmp(dp, "name")) {
		/* We found it! */
		dp[i++] = '\n';
		dp += i;
		i = 0;
		if (r == NULL)
		    /* Only get the name once */
		    searchstate = R_NAME;
	    } else if (!strncmp(dp, "access", 6)) {
		if (strcmp(dp, "access_count"))
		    /* Count?  We don't need no steenkin' count */
		    searchstate = R_ACCESS;
		dp[i++] = '\n';
		dp += i;
		i = 0;
	    } else {
		dp[i++] = '\n';
		dp += i;
		i = 0;
	    }
	}
    }

    free(dest);
    free(buffer);

    if (success == 0)
	return 0;
    return -1;
}
